from flameboi.blocks.block import Block


class TextObject(Block):
    """
    Represents a text object. For more information, see:
    https://api.slack.com/reference/block-kit/composition-objects#text
    """

    BTYPE_PLAIN_TEXT = 'plain_text'
    BTYPE_MARKDOWN = 'mrkdwn'

    def __init__(self, btype: str, text: str, emoji: bool = False, verbatim: bool = False):
        # validate that the type is correct
        if btype == self.BTYPE_PLAIN_TEXT or btype == self.BTYPE_MARKDOWN:
            self.btype = btype
        else:
            raise AttributeError(f'Invalid btype. Must be {self.BTYPE_MARKDOWN} or {self.BTYPE_PLAIN_TEXT}: {btype}')

        self.text = text

        # emoji field is only usable if the type is plain text
        self.emoji = (emoji and btype == self.BTYPE_PLAIN_TEXT)
        self.verbatim = verbatim

    def is_plain_text(self):
        return self.btype == self.BTYPE_PLAIN_TEXT

    def is_markdown(self):
        return self.btype == self.BTYPE_MARKDOWN

    def get_text_length(self):
        return len(self.text)

    def validate_text_block(self, max_length: int = 0, required_type: str = None):
        # check that the text is not greater than the max length
        if max_length != 0 and len(self.text) > max_length:
            raise AttributeError(f'text object text should not be greater than {max_length} characters, but is '
                                 f'{len(self.text)}')
        if required_type and self.btype != required_type:
            raise AttributeError(f'text object type should be {required_type}, but is {self.btype}')

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'text': self.text
        }

        if self.emoji:
            block.update({'emoji': self.emoji})

        if self.verbatim:
            block.update({'verbatim': self.verbatim})

        return block



class Event(object):
    
    def __init__(self, payload):
        self.event = payload.get('event', {})
        self.item = payload.get('event', {}).get('item', {})

    def get_user_id(self):
        return self.event.get('user')

    def get_channel_id(self):
        return self.event.get('channel')

    def get_text(self):
        return self.event.get('text')

    def get_event_ts(self):
        return self.event.get('ts')

    def get_subtype(self):
        return self.event.get('subtype')

    def get_reaction(self):
        return self.event.get('reaction')

    def get_item_channel(self):
        return self.item.get('channel')

    def get_item_ts(self):
        return self.item.get('ts')

    def get_item_file(self):
        return self.item.get('file')

    def get_specs(self) -> dict:
        pass

    # reserved for future implementation



from .block_element import BlockElement


class ImageElement(BlockElement):
    """
    An element to insert an image as part of a larger block of content. If you want a block with only an image in it,
    you're looking for the image block. For more information, see:
    https://api.slack.com/reference/block-kit/block-elements#image
    """

    def __init__(self, image_url: str, alt_text: str):
        super().__init__(btype='image')
        self.image_url = image_url
        self.alt_text = alt_text

    def render(self) -> dict:
        return {
            'type': self.btype,
            'image_url': self.image_url,
            'alt_text': self.alt_text
        }



from flameboi.blocks.layout_block.layout_block import LayoutBlock
from flameboi.blocks.composition_object.text_object import TextObject
from flameboi.blocks.block_element.block_element import BlockElement


class InputBlock(LayoutBlock):
    """
    A block that collects information from users - it can hold a plain-text input element, a select menu element,
    a multi-select menu element, or a datepicker. For more information, see:
    https://api.slack.com/reference/block-kit/blocks#input
    """

    def __init__(self, label: TextObject, element: BlockElement, block_id: str = None, hint: TextObject = None,
                 optional: bool = False):
        # validate input
        label.validate_text_block(max_length=2000, required_type=TextObject.BTYPE_PLAIN_TEXT)
        if hint:
            hint.validate_text_block(max_length=2000, required_type=TextObject.BTYPE_PLAIN_TEXT)

        super().__init__(btype='input', block_id=block_id)

        self.label = label
        self.element = element
        self.hint = hint
        self.optional = optional

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'label': self.label,
            'element': self.element,
            'optional': self.optional
        }

        if self.block_id:
            block.update({'block_id', self.block_id})
        if self.hint:
            block.update({'hint': self.hint})

        return block



from flameboi.blocks.block import Block
from flameboi.blocks.composition_object.text_object import TextObject


class ConfirmObject(Block):
    """
    An object that defines a dialog that provides a confirmation step to any interactive element. This dialog will ask
    the user to confirm their action by offering a confirm and deny buttons. For more information, see:
    https://api.slack.com/reference/block-kit/composition-objects#confirm
    """

    def __init__(self, title: TextObject, text: TextObject, confirm: TextObject, deny: TextObject):
        # validate input
        title.validate_text_block(max_length=100, required_type=TextObject.BTYPE_PLAIN_TEXT)
        text.validate_text_block(max_length=300)
        confirm.validate_text_block(max_length=30, required_type=TextObject.BTYPE_PLAIN_TEXT)
        deny.validate_text_block(max_length=30, required_type=TextObject.BTYPE_PLAIN_TEXT)

        self.title = title
        self.text = text
        self.confirm = confirm
        self.deny = deny

    def render(self) -> dict:
        return {
            'title': self.title.render(),
            'text': self.text.render(),
            'confirm': self.confirm.render(),
            'deny': self.deny.render()
        }



from .block_element import BlockElement
from ..composition_object.text_object import TextObject
from ..composition_object.confirm_object import ConfirmObject


class ButtonElement(BlockElement):
    STYLE_PRIMARY = 'primary'
    STYLE_DANGER = 'danger'
    STYLE_DEFAULT = 'default'

    """
    An interactive component that inserts a button. The button can be a trigger for anything from opening a simple link
    to starting a complex workflow. For more information, see:
    https://api.slack.com/reference/block-kit/block-elements#button
    """

    def __init__(self, text: TextObject, action_id: str, url: str = None, value: str = None, style: str = 'default',
                 confirm: ConfirmObject = None):
        # validate input
        text.validate_text_block(max_length=75, required_type=TextObject.BTYPE_PLAIN_TEXT)
        self.validate_input('url', url, max_length=3000)
        self.validate_input('value', value, max_length=2000)
        self.validate_input('style', style, equality_fields=[self.STYLE_DANGER, self.STYLE_DEFAULT, self.STYLE_PRIMARY])

        super().__init__(btype='button', action_id=action_id)

        self.text = text
        self.url = url
        self.value = value
        self.style = style
        self.confirm = confirm

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'text': self.text.render(),
            'action_id': self.action_id,
        }

        if self.url:
            block.update({'url': self.url})
        if self.value:
            block.update({'value': self.value})
        if self.style:
            block.update({'style': self.style})
        if self.confirm:
            block.update({'confirm': self.confirm.render()})

        return block



from ..common.IEvent import Event


class TeamJoinEvent(Event):

    def __init__(self, payload):
        super().__init__(payload)

    def get_details(self) -> dict:
        return {
            'user_id': self.event.get('user', {}).get('id')
        }



from .block_element import BlockElement
from ..composition_object.text_object import TextObject


class PlainTextInputElement(BlockElement):
    """
    A plain-text input, similar to the HTML <input> tag, creates a field where a user can enter freeform data. It can
    appear as a single-line field or a larger textarea using the multiline flag. For more information, see:
    https://api.slack.com/reference/block-kit/block-elements#input
    """

    def __init__(self, action_id: str, placeholder: TextObject = None, initial_value: str = None,
                 multiline: bool = False, min_length: int = 0, max_length: int = 0):
        # validate input
        if placeholder:
            placeholder.validate_text_block(max_length=150, required_type=TextObject.BTYPE_PLAIN_TEXT)

        if min_length != 0 or max_length != 0:
            if min_length > max_length:
                raise AttributeError(f'max_length ({max_length}) must be greater than min_length ({min_length})')
            if min_length < 0 or min_length > 3000:
                raise AttributeError(f'min_length ({min_length}) must be between 0 and 3000 (inclusive)')
            if max_length < 0 or max_length > 3000:
                raise AttributeError(f'max_length ({max_length}) must be between 0 and 3000 (inclusive)')

        super().__init__(btype='plain_text_input', action_id=action_id)

        self.placeholder = placeholder
        self.initial_value = initial_value
        self.multiline = multiline
        self.min_length = min_length
        self.max_length = max_length

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'action_id': self.action_id,
            'multiline': self.multiline
        }

        if self.placeholder:
            block.update({'placeholder': self.placeholder.render()})
        if self.initial_value:
            block.update({'initial_value': self.initial_value})
        if self.min_length != 0:
            block.update({'min_length': self.min_length})
        if self.max_length != 0:
            block.update({'max_length': self.max_length})

        return block



'''
TODO: implement this using the newly modular setup
'''

import os
import json
import urllib.request

from base64 import b64decode
from urllib.parse import urlparse

EXPECTED_TOKEN = os.environ['VERIFICATION_TOKEN']

def respond(err, res=None):
    return {
        'statusCode': '400' if err else '200',
        'headers': {
            'Content-Type': 'application/json',
        },
        'body': err.message if err else json.dumps({
            'text': res,
            'response_type': 'in_channel'
        })
    }


def lambda_handler(event, context):
    
    params = urllib.parse.parse_qs(b64decode(event['body']))
    
    token = params[b'token'][0].decode("utf-8")
    
    if token != EXPECTED_TOKEN:
        return respond(Exception('Invalid request token'))
        
    count = -1
    output = ""
    
    url = "https://apps.foldingathome.org/daily_team_summary.txt"

    file = urllib.request.urlopen(url)

    for line in file:
        decoded_line = line.decode("utf-8")
        if "CodeDevils" in decoded_line:
            output = decoded_line
            break
        count += 1
        
    if not output:
        return respond(None, "CodeDevils not found!")
    
    data = output.split('\t', 3)

    return respond(None, f"The CodeDevils are currently ranked #{count:,} with {int(data[2]):,} points amassed!\nFor a full breakdown of our team go to https://stats.foldingathome.org/team/249718!")



import os
from dotenv import load_dotenv
from slack import WebClient
from slackeventsapi import SlackEventAdapter

from ..blocks.block_generator import BlockGenerator


class Flameboi:
    """
    Methods used to control various features with the Flameboi Slack bot API.

        Args:
            app (Flask obj): the Flask object being used.  Initiated in app.py
            SLACK_SIGNING_SECRET (str): Signing secret stored in dotenv file in the /src directory.  See 
                server_configs/dotenvsample.txt for layout.
            SLACK_BOT_TOKEN (str): Bot User Token secret stored in dotenv file in the /src directory.  See 
                server_configs/dotenvsample.txt for layout.
    """

    def __init__(self, app):
        load_dotenv()
        self.signing_secret = os.getenv("SLACK_SIGNING_SECRET")
        self.bot_token = os.getenv("SLACK_BOT_TOKEN")

        self.blockGen = BlockGenerator()
        self.bot_client = WebClient(token=self.bot_token)
        self.event_adapter = SlackEventAdapter(self.signing_secret, "/", app)

    def getClient(self) -> WebClient:
        """
        Returns the slack web_client.

        :return: The slack client.
        :rtype: slack.WebClient
        """
        return self.bot_client

    def getAdapter(self) -> SlackEventAdapter:
        """
        Returns the slack event adapter.

        :return: The slack client.
        :rtype: slack.WebClient
        """
        return self.event_adapter

    def add_member(self, user_email: str) -> dict:
        """
        This method does not work yet. Enterprise grid access is required for the app to be able to add/remove users
        from the team.

        Adds a member to the workspace using their member ID.

        :param user_email: The Slack ID of the user.
        :type user_email: str
        :return: The response of the member addition request.
        :rtype: dict
        """
        user = self.get_user_by_email(email=user_email)['user']
        channel = self.get_channel_id(channel_name='hangout')
        return self.user_client.channels_invite(channel=channel, user=user['id'])

    def get_user_by_email(self, email: str) -> dict:
        """
        Retrieves a user by their email.

        :param email: The email address of the user.
        :type email: str
        :return: The user as a dict.
        :rtype: dict
        """
        return self.bot_client.users_lookupByEmail(email=email)

    def get_users_list(self) -> dict:
        """
        Returns a list of users.

        :return: A list of users. See https://api.slack.com/methods/users.list for format.
        :rtype: dict
        """
        return self.bot_client.users_list()

    def get_channel_id(self, channel_name: str) -> str:
        """
        Returns the channel ID based on the name. If no ID is found, then none is returned.

        :param channel_name: The name of the channel as a string (w/o the leading hashtag).
        :type channel_name: str
        :return: The channel id as a string.
        :rtype: str
        """
        channel_list = self.get_channel_list()
        for channel in channel_list:
            if channel['name'] == channel_name:
                return channel['id']

        return str(None)

    def get_user_info(self, user_id: str) -> dict:
        """
        Returns the information about the user, identified by user_id.

        :param user_id: The ID of the user as a string (w/o the leading @).
        :return: The user's info as a dict.
        :rtype: dict
        """
        return self.bot_client.users_info(user=user_id)

    def get_channel_list(self) -> dict:
        """
        Returns the list of channels available to the bot.

        :return: The list of channels as a dict.
        :rtype: dict
        """
        return self.bot_client.channels_list()

    def send_message(self, channel: str, text: str, mention_email: str = None) -> dict:
        """
        Sends a message (either text or block) to a channel. An optional mention can be added to the beginning of the
        message.

        :param channel: The name of the channel to send the message to.
        :type channel: str
        :param text: The message to send.
        :type text: str
        :param mention_email: The email of the user to mention, default is None.
        :type mention_email: str
        :return: The result of sending the message.
        """

        if mention_email:
            username = self.get_user_by_email(mention_email)['user']['name']
            text = '@{} {}'.format(username, text)
        message = self.blockGen.get_message_payload(text=text, channel=channel)
        return self._send_block_message(message=message)

    def _send_block_message(self, message: dict, user_id: int = 0) -> dict:
        """
        Block message util to send a message using the bot client.

        :param message: Markdown-supported message to be sent.
        :type message: dict
        :param user_id: The user ID (if any) of the user the message was sent to.
        :type user_id: int, optional
        :return: The response from sending the message as a dict.
        :rtype: dict
        """
        return self.bot_client.chat_postMessage(**message)



from .block_element import BlockElement
from ..composition_object.option_object import OptionObject
from ..composition_object.option_group_object import OptionGroupObject
from ..composition_object.confirm_object import ConfirmObject


class RadioButtonGroupElement(BlockElement):
    """
    A radio button group that allows a user to choose one item from a list of possible options. For more information,
    see: https://api.slack.com/reference/block-kit/block-elements#radio
    """

    def __init__(self, action_id: str, options: [OptionObject], intitial_option: OptionObject = None,
                 confirm: ConfirmObject = None):
        # validate input
        if intitial_option and intitial_option not in options:
            raise AttributeError('initial_option must be an option within options')
        super().__init__(btype='radio_buttons', action_id=action_id)

        self.options = options
        self.initial_option = intitial_option
        self.confirm = confirm

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'action_id': self.action_id,
            'options': OptionGroupObject.expand_options_group(self.options)
        }

        if self.initial_option:
            block.update({'initial_option': self.initial_option.render()})
        if self.confirm:
            block.update({'confirm': self.confirm.render()})

        return block



from flameboi.blocks.layout_block.layout_block import LayoutBlock


class ContextBlock(LayoutBlock):
    """
    Displays message context, which can include both images and text. For more information, see:
    https://api.slack.com/reference/block-kit/blocks#context
    """

    def __init__(self, elements: list, block_id: str = None):
        # validate input
        if len(elements) > 10:
            raise AttributeError('cannot have more than 10 elements in context blocks')

        super().__init__(btype='context', block_id=block_id)
        self.elements = elements

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'elements': self.elements
        }

        if self.block_id:
            block.update({'block_id': self.block_id})

        return block



'''
TODO: implement this using the newly modular setup
'''

from flameboi.blocks.block_generator import BlockGenerator

def get_welcome_block(self, channel: str) -> dict:
    """
    Sends the welcome message to the user.

    :param channel: The channel ID of the user as a string.
    :type channel: str
    :return: The response from the message payload.
    :rtype: dict
    """
    self.channel = channel
    return self.get_message_payload(blocks=get_onboarding_block())

def send_onboarding_message(self, user_email: str) -> dict:
    """
    Sends the onboarding message to a user.

    :param user_email: The email of the user to be on-boarded.
    :type user_email: str
    :return: The response from the message request as a dict.
    :rtype: dict
    """
    user = self.get_user_by_email(email=user_email)['user']['id']
    response = self.bot_client.conversations_open(users=[user])
    if not response['ok']:
        return response

    channel = response['channel']['id']
    message = self.messenger.get_welcome_block(channel=channel)
    return self._send_block_message(message=message)


def send_onboarding_DM(self, user_id: str) -> dict:
    """
    Sends the onboarding message to a user.

    :param user_id: The slack ID of the user to be on-boarded.
    :type user_id: str
    :return: The response from the message request as a dict.
    :rtype: dict
    """
    response = self.bot_client.conversations_open(users=[user_id])
    if not response['ok']:
        return response

    channel = response['channel']['id']
    message = self.messenger.get_welcome_block(channel=channel)
    return self._send_block_message(message=message)

def get_onboarding_block() -> list:
    """
    Constructs an onboarding block, which contains information on different channels and a link to the CodeDevils
    website.

    :return: The onboarding message block as a list.
    :rtype: list
    """

    block = [
        SectionBlock(text=TextObject(btype=TextObject.BTYPE_MARKDOWN,
                                        text="Welcome to *CodeDevils*! I'm Flameboi, and I'll help you get settled. C"
                                            "heck out some of the channels available here:\n\n*Channels:*")).render(),
        DividerBlock().render(),
        get_text_block_with_image(text="*Opportunities*\nLearn about exciting new job opportunities and internships"
                                        " from other members, and gain insight on how to succeed during interviews.",
                                    image_url="https://slack-imgs.com/?c=1&url=https%3A%2F%2Fcdn0.iconfinder.com%2Fdata%2"
                                            "Ficons%2Feducation-340%2F100%2FTilda_Icons_1ed_cup-256.png",
                                    alt_text="Opportunities"),
        get_text_block_with_image(text="*Project Discussion*\nEver think about programming something real and "
                                        "usable with a team? Use this channel to join a project, discuss one, or "
                                        "even get one started.",
                                    image_url="https://slack-imgs.com/?c=1&url=https%3A%2F%2Fcdn0.iconfinder.com%2Fdata%2"
                                            "Ficons%2Feducation-340%2F100%2FTilda_Icons_1ed_group-256.png",
                                    alt_text="Project Discussion"),
        get_text_block_with_image(text="*Random*\nPost literally anything that you want! College is too boring to "
                                        "be serious all the time, so brighten someone's day up with a random thought"
                                        " or funny meme.",
                                    image_url="https://slack-imgs.com/?c=1&url=https%3A%2F%2Fcdn0.iconfinder.com%2Fdata%2"
                                            "Ficons%2Feducation-340%2F99%2FTilda_Icons_1ed_speaker-256.png",
                                    alt_text="Random"),
        DividerBlock().render(),
        get_text_block_with_image(text="*CodeDevils Website*\nDid you know that I'm powered by the CodeDevils "
                                        "website? The website is so much more than just a web page. It's a web "
                                        "application used to communicate with fellow members, register members to "
                                        "CodeDevils, and coordinate on projects.\n\nVisit the "
                                        "<https://www.codedevils.org|CodeDevils website> to learn more.",
                                    image_url="https://www.codedevils.org/static/home/img/favicon.png",
                                    alt_text="CodeDevils Website"),
        DividerBlock().render()
    ]
    print(f'{block}')
    return block


import os
from ..events.app_mention_event import AppMentionEvent
from ..events.channel_join_event import ChannelJoinEvent
from ..events.message_event import MessageEvent
from ..events.pin_added_event import PinAddedEvent
from ..events.reaction_added_event import ReactionAddedEvent
from ..events.team_join_event import TeamJoinEvent
# from ..events.slash_command import SlashCommand


class Router:
    """
    Methods used to router/direct events within the Flameboi Slack bot API.

    :return: The list of channels as a dict.
    :rtype: dict
    """

    def __init__(self, bot_client):
        self.bot = bot_client

        # Import various ID's for filtering via dotenv
        self.bot_id = os.getenv("BOT_ID")
        self.bot_user_id = os.getenv("USER_ID")
        self.bot_app_id = os.getenv("APP_ID")

    # TODO: implement this
    def handle_team_join(self, payload):
        """
        Returns the list of channels available to the bot.

        :return: The list of channels as a dict.
        :rtype: dict
        """

        event = TeamJoinEvent(payload)

    # TODO: implement this
    def handle_reaction_added(self, payload):

        """
        Returns the list of channels available to the bot.

        :return: The list of channels as a dict.
        :rtype: dict
        """

        event = ReactionAddedEvent(payload)
        details = event.get_details()

        if details['user_id'] != self.bot_user_id:
            response = self.bot.reactions_add(
                name=details['reaction'],
                channel=details['channel_id'],
                timestamp=details['ts']
            )
            assert response["ok"]

    # TODO: implement this
    def handle_pin_added(self, payload):
        """
        Returns the list of channels available to the bot.

        :return: The list of channels as a dict.
        :rtype: dict
        """

        event = PinAddedEvent(payload)
        details = event.get_details()

        reply = f"<@{details['user_id']}> seems to think something of importance happened in <@{details['channel_id']}>"

        assert self.bot.send_message('C30L07P18', reply)["ok"]

    # TODO: implement this
    def handle_message(self, payload):
        """
        Returns the list of channels available to the bot.

        :return: The list of channels as a dict.
        :rtype: dict
        """

        event = MessageEvent(payload)
        details = event.get_details()

        if details['sub_type'] != 'bot_message':
            if details['text'] and details['text'].lower() == "!test":
                reply = ":tada: :partywizard: I'm here <@%s>! :partywizard: :tada:" % details['user_id']

                assert self.bot.send_message(details['channel_id'], reply)["ok"]

            if "party" in details['text'] and ":partywizard:" not in details['text']:
                reply = ":partywizard:"

                assert self.bot.send_message(details['channel_id'], reply)["ok"]

            """
            TODO: Expand on block kit builder base (which is awesome Kevin!)
            Below is example use of blocks using !onboard to send the onboarding block
            """

            # if details['text'] and details['text'].lower() == "!onboard":
            #     assert self.bot.send_onboarding_DM(details['user_id'])["ok"]
            #
            # if details['text'] and details['text'].lower() == "!qod":
            #     assert self.bot.send_qod(details['channel_id'])["ok"]

    # TODO: implement this
    def handle_channel_join(self, payload):
        """
        Returns the list of channels available to the bot.

        :return: The list of channels as a dict.
        :rtype: dict
        """

        event = ChannelJoinEvent(payload)
        details = event.get_details()

        reply = f"Welcome to <@{details['channel_id']}>, <@{details['user_id']}>!!"

        assert self.bot.send_message(details['channel_id'], reply)["ok"]

    # TODO: implement this
    def handle_app_mention(self, payload):
        """
        Returns the list of channels available to the bot.

        :return: The list of channels as a dict.
        :rtype: dict
        """

        event = AppMentionEvent(payload)
        details = event.get_details()

        reply = f"You talking to me, <@{details['user_id']}>?!@?"
        assert self.bot.send_message(details['channel_id'], reply)["ok"]

    # # TODO: implement this
    # def handle_slash_command(self, payload):
    #     """
    #     Returns the list of channels available to the bot.
    #
    #     :return: The list of channels as a dict.
    #     :rtype: dict
    #     """



from ..common.IEvent import Event

class ChannelJoinEvent(Event):

    def __init__(self, payload):
        super().__init__(payload)

    def get_details(self) -> dict:
        return {
            'user_id': self.get_user_id,
            'channel_id': self.get_channel_id
        }



'''
TODO: implement this using the newly modular setup
'''

import requests
from flameboi.blocks.block_generator import BlockGenerator

def getQOD() -> list:

    output = []

    url = 'https://quotes.rest/qod'
    api_token = "YOUR API KEY HERE"
    headers = {'content-type': 'application/json',
        'X-TheySaidSo-Api-Secret': format(api_token)}

    response = requests.get(url, headers=headers)
    #print(response)
    output.append(response.json()['contents']['quotes'][0]['quote'])
    output.append(response.json()['contents']['quotes'][0]['author'])

    return output


def get_quote(self, channel: str) -> dict:
    """
    Send a quote of the day to the specified channel.

    :param channel: The channel ID as a string.
    :type channel: str
    :return: The response from the message payload.
    :rtype: dict
    """
    self.channel = channel
    return self.get_message_payload(blocks=get_qod_block())


def send_qod(self, channel_id) -> dict:
    """
    Sends the quote of the day to specified channel.

    :param channel_id: The channel ID.
    :type channel_id: str
    :return: The response from the message request as a dict.
    :rtype: dict
    """
    message = self.messenger.get_quote(channel=channel_id)
    return self._send_block_message(message=message)

def get_qod_block() -> list:
    """
    Constructs a quote of the day block.

    :return: The quote of the day block as a list.
    :rtype: list
    """

    quote = getQOD()

    block = [
                get_text_block_with_image(text="*Quote of the Day*\n%s\n\t~~%s~~" % (quote[0], quote[1]),
                                        image_url="https://media.giphy.com/media/3og0IMJcSI8p6hYQXS/giphy.gif",
                                        alt_text="QOD")
            ]

    print(f'{block}')
    return block


from flameboi.blocks.layout_block.layout_block import LayoutBlock


class FileBlock(LayoutBlock):
    """
    Displays a remote file. For more information, see:
    https://api.slack.com/reference/block-kit/blocks#file
    """

    def __init__(self, external_id: str, source: str = 'remote', block_id: str = None):
        super().__init__(btype='file', block_id=block_id)
        self.external_id = external_id
        self.source = source

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'external_id': self.external_id,
            'source': self.source
        }

        if self.block_id:
            block.update({'block_id': self.block_id})

        return block



import os
from dotenv import load_dotenv
from datetime import datetime
from ..blocks.block_element.block_element import BlockElement
from ..blocks.composition_object.text_object import TextObject
from ..blocks.layout_block.section_block import SectionBlock
from ..blocks.block_element.image_element import ImageElement
from ..blocks.layout_block.divider_block import DividerBlock


class BlockGenerator:
    """
    Constructs block payloads that act as formatted messages for Slack. For more information on blocks, see
    https://api.slack.com/block-kit.
    """

    def __init__(self, channel=None):
        """
        Initializes the payload parameters to be sent with each message.

        :param config: The configuration as a dict.
        :type config: dict
        :param channel: The default channel. If not specified, it takes the default channel from the
            config, defaults to None
        :type channel: str, optional
        """
        self.channel = channel if channel else os.getenv("DEFAULT_CHANNEL")
        self.username = os.getenv("USERNAME")
        self.icon_emoji = os.getenv("ICON_EMOJI")
        self.icon_url = os.getenv("ICON_URL")

    def get_message_payload(self, ts: datetime = None, text: str = None, channel: str = None, blocks: list = None,
                            view_type: str = None, link_names: int = 1) -> dict:
        """
        Returns a message block payload used to post messages in a specific channel.

        :param blocks: The blocks to be inserted into the message payload.
        :type blocks: list
        :param channel: channel_id
        :type channel: string
        :param ts: datetime
        :type ts: string
        :param text: channel_id
        :type text: string
        :param link_names: number of linke names, defaults to 1
        :type link_names: int
        :param view_type: Used to determine the type of block for a view publish,
            defaults to None
        :type view_type: str, optional
        :return: The message payload as a dict.
        :rtype: dict
        """
        payload = {
            "ts": str(ts if ts else datetime.now()),
            "channel": channel if channel else self.channel,
            "username": self.username,
            "icon_emoji": self.icon_emoji,
            "icon_url": self.icon_url,
        }
        payload.update({'text': text}) if text else payload.update({'blocks': blocks})
        payload.update({'link_names': link_names}) if link_names == 1 else payload.update({'link_names': 0})

        if view_type:
            payload.update({'type': view_type})

        return payload

    def get_checkmark(self, task_completed: bool) -> str:
        """
        Returns a check mark emoji indicating the completed task status. If the task is complete, then a white check
        mark is returned. If not, an empty white square is.

        :param task_completed: Whether or not the task was complete.
        :type task_completed: bool
        :return: A checkmark emoji string based on whether or not the task was completed.
        :rtype: str
        """
        if task_completed:
            return ":white_check_mark:"
        return ":white_large_square:"

    def get_information_block(self, info_link: str, info_text: str) -> dict:
        """
        Returns an information block, which is a section with an info icon followed by linked text.

        :param info_link: The link the block redirects the user to.
        :type info_link: str
        :param info_text: The link text.
        :type info_text: str
        :return: A dict in the format of a context block.
        :rtype: dict
        """
        information = f':information_source: *<{info_link}|{info_text}>*'
        return TextObject(btype=TextObject.BTYPE_MARKDOWN, text=information).render()

    def get_text_block_with_accessory(self, text_object: TextObject, accessory: BlockElement) -> dict:
        """
        Returns a text block with an accessory.

        :param text_object: The text block object.
        :type text_object: TextObject
        :param accessory: The accessory object.
        :type accessory: LayoutBlock
        :return: The text block with an accessory layout block.
        :rtype: dict
        """
        return SectionBlock(text=text_object, accessory=accessory).render()

    def get_text_block_with_image(self, text: str, image_url: str, alt_text: str) -> dict:
        """
        Returns a text block with an image to the right of it.

        :param text: The text in the text block.
        :type text: str
        :param image_url: The URL to the image.
        :type image_url: str
        :param alt_text: Alternate text (appears on image hover).
        :type alt_text: str
        :return: The block as a dict.
        :rtype: dict
        """
        text_object = TextObject(btype=TextObject.BTYPE_MARKDOWN, text=text)
        image_element = ImageElement(image_url=image_url, alt_text=alt_text)
        return self.get_text_block_with_accessory(text_object=text_object, accessory=image_element)

    def get_task_block(self, text: str, info_link: str, info_text: str) -> list:
        """
        Returns a task block, which is comprised of a paragraph of text followed by an information link at the bottom.

        :param text: Markdown-supported text to display in the paragraph.
        :type text: str
        :param info_link: The link associated with the task block.
        :type info_link: str
        :param info_text: The link text.
        :type info_text: str
        :return: An array of blocks formatted for a block payload.
        :rtype: list
        """
        return [
            TextObject(btype=TextObject.BTYPE_MARKDOWN, text=text).render(),
            self.get_information_block(info_link=info_link, info_text=info_text)
        ]



from datetime import date

from .block_element import BlockElement
from ..composition_object.text_object import TextObject
from ..composition_object.confirm_object import ConfirmObject


class DatepickerElement(BlockElement):
    """
    An element which lets users easily select a date from a calendar style UI. For more information, see:
    https://api.slack.com/reference/block-kit/block-elements#datepicker
    """

    def __init__(self, action_id: str, placeholder: TextObject, initial_date: str = None,
                 confirm: ConfirmObject = None):
        # validate input
        placeholder.validate_text_block(max_length=150, required_type=TextObject.BTYPE_PLAIN_TEXT)

        # set date if not already set
        if not initial_date:
            initial_date = str(date.today())

        super().__init__(btype='datepicker', action_id=action_id)

        self.placeholder = placeholder
        self.initial_date = initial_date
        self.confirm = confirm

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'action_id': self.action_id,
            'placeholder': self.placeholder.render(),
            'initial_date': self.initial_date
        }

        if self.confirm:
            block.update({'confirm': self.confirm.render()})

        return block



from flameboi.blocks.block import Block
from flameboi.blocks.composition_object.text_object import TextObject


class OptionObject(Block):
    """
    An object that represents a single selectable item in a select menu, multi-select menu, radio button group,
    or overflow menu. For more information, see:
    https://api.slack.com/reference/block-kit/composition-objects#option
    """

    def __init__(self, text: TextObject, value: str, url: str = None):
        # validate input
        text.validate_text_block(max_length=75, required_type=TextObject.BTYPE_PLAIN_TEXT)
        self.validate_input('value', value, max_length=75)
        self.validate_input('url', url, max_length=3000)

        self.text = text
        self.value = value
        self.url = url

    def render(self) -> dict:
        block = {
            'text': self.text.render(),
            'value': self.value
        }

        if self.url:
            block.update({'url': self.url})

        return block



from flameboi.blocks.layout_block.layout_block import LayoutBlock
from flameboi.blocks.composition_object.text_object import TextObject
from flameboi.blocks.block_element.block_element import BlockElement


class SectionBlock(LayoutBlock):
    """
    Basic section block. For more information, see: https://api.slack.com/reference/block-kit/blocks#section
    """

    def __init__(self, text: TextObject, block_id: str = None, fields: list = None, accessory: BlockElement = None):
        super().__init__(btype='section', block_id=block_id)

        # field validation
        # text can be no longer than 3000 characters
        if text.get_text_length() > 3000:
            raise AttributeError(f'text cannot be more than 3000 characters, but got {text.get_text_length()}')

        if block_id and len(block_id) > 255:
            raise AttributeError(f'block_id cannot be more than 255 characters, but got {len(block_id)}')

        self.text = text
        self.block_id = block_id
        self.fields = fields
        self.accessory = accessory

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'text': self.text.render(),
        }

        if self.block_id:
            block.update({'block_id': self.block_id})
        if self.fields:
            block.update({'fields': self.fields})
        if self.accessory:
            block.update({'accessory': self.accessory.render()})

        return block



from flameboi.blocks.block import Block
from flameboi.blocks.composition_object.text_object import TextObject


class ViewPayload(Block):
    """
    Defines a view payload. For more information, see: https://api.slack.com/reference/surfaces/views
    """

    BTYPE_MODAL = 'modal'
    BTYPE_HOME = 'home'

    def __init__(self, btype: str, title: TextObject, blocks: list,
                 close: TextObject = None, submit: TextObject = None, private_metadata: str = None,
                 callback_id: str = None, clear_on_close: bool = False, notify_on_close: bool = False,
                 external_id: str = None):

        # type validation
        # type must be either home or modal
        if btype != self.BTYPE_HOME and btype != self.BTYPE_MODAL:
            raise AttributeError(f'Invalid btype. Type must be {self.BTYPE_HOME} or {self.BTYPE_MODAL}: {btype}')

        # close must be plain text and 24 characters or less
        if close:
            if not close.is_plain_text():
                raise AttributeError('close must be plain text')
            if close.get_text_length() > 24:
                raise AttributeError(f'close text must be 24 characters or less, but length '
                                     f'is {close.get_text_length()}')

        # submit must be plain text, 24 characters or less, and must exist if there is an input block within the blocks
        # array
        if submit:
            if not submit.is_plain_text():
                raise AttributeError('submit must be plain text')
            if submit.get_text_length() > 24:
                raise AttributeError(f'submit text must be 24 characters or less, but length '
                                     f'is {submit.get_text_length()}')

        # private_metadata must be 3000 characters or less
        if private_metadata and len(private_metadata) > 3000:
            raise AttributeError(f'private_metadata must be 3000 characters or less. Currently {len(private_metadata)}')

        # callbacl_id must be 255 characters or less
        if callback_id and len(callback_id) > 255:
            raise AttributeError(f'callback_id must be 3000 characters or less. Currently {len(callback_id)}')

        self.btype = btype
        # titles are used only for modals
        if self.btype == self.BTYPE_MODAL:
            self.title = title
        self.blocks = blocks
        self.close = close
        self.submit = submit
        self.private_metadata = private_metadata
        self.callback_id = callback_id
        self.clear_on_close = clear_on_close
        self.notify_on_close = notify_on_close
        self.external_id = external_id

    def render(self) -> dict:
        # required parameters
        block = {
            'type': self.btype,
            'title': self.title.render(),
            'blocks': self.blocks,
        }

        # optional parameters
        if self.close:
            block.update({'close': self.close.render()})
        if self.submit:
            block.update({'submit': self.submit.render()})
        if self.private_metadata:
            block.update({'private_metadata': self.private_metadata})
        if self.callback_id:
            block.update({'callback_id': self.callback_id})
        if not self.clear_on_close:
            block.update({'clear_on_close': self.clear_on_close})
        if not self.notify_on_close:
            block.update({'notify_on_close': self.notify_on_close})
        if self.external_id:
            block.update({'external_id': self.external_id})

        return block




from flask import Flask
from .common.routing import Router
from .common.flameboi import Flameboi


# Initialize a Flask app to host the events adapter
app = Flask(__name__)

# Initialize a Web API client, Router and Slack Events adapter
theBot = Flameboi(app)
slack_web_client = theBot.getClient()
slack_events_adapter = theBot.getAdapter()
router = Router(slack_web_client)


# ================ Team Join Event =============== #

@slack_events_adapter.on("team_join")
def onboarding_message(payload):
    """Create and send an onboarding welcome message to new users. Save the
    time stamp of this message so we can update this message in the future.
    """
    
    router.handle_team_join(payload)


# ============= Reaction Added Events ============= #

@slack_events_adapter.on("reaction_added")
def update_emoji(payload):
    """Update the onboarding welcome message after receiving a "reaction_added"
    event from Slack. Update timestamp for welcome message as well.
    """

    router.handle_reaction_added(payload)


# =============== Pin Added Events ================ #

@slack_events_adapter.on("pin_added")
def update_pin(payload):
    """Update the onboarding welcome message after receiving a "pin_added"
    event from Slack. Update timestamp for welcome message as well.
    """

    router.handle_pin_added(payload)


# ============== Message Events ============= #

@slack_events_adapter.on("message")
def message(payload):
    """Display the onboarding welcome message after receiving a message
    that contains "start".
    """

    router.handle_message(payload)


# ============== Member Join Events ============= #

@slack_events_adapter.on("member_joined_channel")
def member_joined(payload):
    """Display the channel welcome message after someone joins a channel.
    """
    
    router.handle_channel_join(payload)


# ============== App Mention Events ============= #

@slack_events_adapter.on("app_mention")
def mention(payload):
    """
    Triggers handler for when the bot received an @ mention event..
    """

    router.handle_app_mention(payload)


# ============== Other Endpoints ============= #


# @app.route('/slash/listener')
# def slashcommand():
#     """
#     Triggers handler for when the bot received slash command..
#     """
#     router.handle_slash_command()


# @app.route('/newuser')
# def addNewUser():
#     """
#     Triggers slackbot to add new user to CodeDevils Slack (secured by jwt) using
#     grid approved email
#     """
#     #TODO: JWT Secured endpoint for web to add user


if __name__ == "__main__":

    app.run(port=5000)



from abc import ABC


class Block(ABC):
    def render(self) -> dict:
        """
        Renders a block as a dict. The formatting is used to insert into a message payload.

        :return: The block as a dict.
        :rtype: dict
        """
        pass

    @staticmethod
    def validate_input(input_name: str, input_value, max_length: int = 0, equality_fields: list = None):
        """
        Validates an input using a max length and equality field constraint. If either are not met, then
        this method raises an attribute error.

        :param input_name: The name of the input field. Used to display with the attribute error message.
        :type input_name: str
        :param input_value: The value of the input.
        :param max_length: The max length the input value can be, defaults to 0. If 0, max length is not validated.
        :type max_length: int
        :param equality_fields: A list of fields that the input value must be equal to, defaults to None. If not
            specified, the validation is not made.
        :type equality_fields: list
        :return: None
        :except AttributeError: Raised if the criteria is not met.
        """

        if input_value and isinstance(input, str):
            # if max length is specified, check that the value does not exceed the length
            if max_length != 0 and len(input_value) > max_length:
                raise AttributeError(f'{input_name} cannot be greater than {max_length} characters, '
                                     f'but it {len(input_value)}')

            # if equality fields are specified, check that the string equals one of those fields
            if equality_fields and input_value not in equality_fields:
                raise AttributeError(f'{input_name} needs to be one of the following values: '
                                     f'{",".join(equality_fields)}')



'''
TODO: implement this using the newly modular setup
'''

import os
import random
import time


def roll(limit):
    
    random.seed(time.time())

    output = 0

    if limit:
        
        stripped = limit.split(' ', 1)

        try:
            tmp = int(stripped[0])
        except:
            return ", a %s is not an integer..." % stripped[0]
        
        output = random.randint(0,tmp)

    else:
        output = random.randint(0,100)

    return " rolled a %d" % output


from .block_element import BlockElement
from ..composition_object.option_object import OptionObject
from ..composition_object.option_group_object import OptionGroupObject
from ..composition_object.confirm_object import ConfirmObject


class OverflowElement(BlockElement):
    """
    This is like a cross between a button and a select menu - when a user clicks on this overflow button, they will be
    presented with a list of options to choose from. Unlike the select menu, there is no typeahead field, and the
    button always appears with an ellipsis ("…") rather than customisable text. For more information, see:
    https://api.slack.com/reference/block-kit/block-elements#overflow
    """

    def __init__(self, action_id: str, options: [OptionObject], confirm: ConfirmObject = None):
        # validate input
        if len(options) > 5 or len(options) < 2:
            raise AttributeError('Must have between 2 and 5 options (inclusive)')
        super().__init__(btype='overflow', action_id=action_id)
        self.options = options
        self.confirm = confirm

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'action_id': self.action_id,
            'options': OptionGroupObject.expand_options_group(self.options),
        }

        if self.confirm:
            block.update({'confirm': self.confirm.render()})

        return block



from flameboi.blocks.layout_block.layout_block import LayoutBlock
from flameboi.blocks.composition_object.text_object import TextObject


class ImageBlock(LayoutBlock):
    """
    A simple image block, designed to make those cat photos really pop. For more information, see:
    https://api.slack.com/reference/block-kit/blocks#image
    """

    def __init__(self, image_url: str, alt_text: str, title: TextObject = None, block_id: str = None):
        # validate input
        self.validate_input('image_url', image_url, max_length=3000)
        self.validate_input('alt_text', alt_text, max_length=2000)
        if title:
            title.validate_text_block(max_length=200, required_type=TextObject.BTYPE_PLAIN_TEXT)

        # initialize the parent
        super().__init__(btype='image', block_id=block_id)

        # set values once validation has passed
        self.image_url = image_url
        self.alt_text = alt_text
        self.title = title

    def render(self) -> dict:
        block = {
            'type': self.btype,
            'image_url': self.image_url,
            'alt_text': self.alt_text
        }

        # add optional fields if specified
        if self.title:
            block.update({'title': self.title.render()})
        if self.block_id:
            block.update({'block_id': self.block_id})

        return block



from ..block import Block


class BlockElement(Block):
    """
    Defines a basic block element. For more information, see: https://api.slack.com/reference/block-kit/block-elements
    """

    def __init__(self, btype: str, action_id: str = None):
        # validate the action id
        if action_id and len(action_id) > 255:
            raise AttributeError(f'action_id cannot be more than 255 characters long, but is {len(action_id)}')

        self.btype = btype
        self.action_id = action_id

    def render(self) -> dict:
        block = {
            'type': self.btype
        }

        if self.action_id:
            block.update({'action_id': self.action_id})

        return block



import requests
import json

def lambda_handler(event, context):
    
    
    url = "https://covid-193.p.rapidapi.com/statistics"
    
    querystring = {"country":"all"}
    querystring2 = {"country":"USA"}
    
    headers = {
        'x-rapidapi-host': "covid-193.p.rapidapi.com",
        'x-rapidapi-key': "556658f588mshae9e612c19896b6p181723jsnb7e9ebd5dfbf"
        }
    
    response = requests.request("GET", url, headers=headers, params=querystring)
    response2 = requests.request("GET", url, headers=headers, params=querystring2)
    
    res = response.json()
    res2 = response2.json()
    
    usCases = res2['response'][0]['cases']
    usDeaths = res2['response'][0]['deaths']
    allCases = res['response'][0]['cases']
    allDeaths = res['response'][0]['deaths']
    
    
    ans = "Current Covid-19 Stats\nWorldwide\n\tCases:\n\t\tActive: {}\n\t\tNew: {}\n\t\tCritical: {}\n\t\tRecovered: {}\n\t\tTotal: {}\n\tDeaths: \n\t\tNew: {}\n\t\tTotal: {}\n".format(allCases['active'], allCases['new'], allCases['critical'], allCases['recovered'], allCases['total'], allDeaths['new'], allDeaths['total'])
    ans += "\nUSA\n\tCases:\n\t\tActive: {}\n\t\tNew: {}\n\t\tCritical: {}\n\t\tRecovered: {}\n\t\tTotal: {}\n\tDeaths:\n\t\tNew: {}\n\t\tTotal: {}".format(usCases['active'], usCases['new'], usCases['critical'], usCases['recovered'], usCases['total'], usDeaths['new'], usDeaths['total'])
        
    ansblock = [
    		{
    			"type": "section",
    			"fields": [
    				{
    					"type": "mrkdwn",
    					"text": "*Worldwide*"
    				},
    				{
    					"type": "mrkdwn",
    					"text": "*USA*"
    				}
    			]
    		},
    		{
    			"type": "divider"
    		},
    		{
    			"type": "section",
    			"fields": [
    								{
    					"type": "mrkdwn",
    					"text": "Active Cases: %s" % allCases['active']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Active Cases: %s" % usCases['active']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "New Cases: %s" % allCases['new']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "New Cases: %s" % usCases['new']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Critical Cases: %s" % allCases['critical']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Critical Cases: %s" % usCases['critical']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Recovered Cases: %s" % allCases['recovered']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Recovered Cases: %s" % usCases['recovered']
    				},
                    {
    					"type": "mrkdwn",
    					"text": "Total Cases: %s" % allCases['total']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Total Cases: %s" % usCases['total']
    				}
    			]
    		},
    		{
    			"type": "divider"
    		},
    		{
    			"type": "section",
    			"fields": [
    				{
    					"type": "mrkdwn",
    					"text": "New Deaths: %s" % allDeaths['new']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "New Deaths: %s" % usDeaths['new']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Total Deaths: %s" % allDeaths['total']
    				},
    				{
    					"type": "mrkdwn",
    					"text": "Total Deaths: %s" % usDeaths['total']
    				}
    			]
    		}
    	]
    
        
    return {
        'statusCode': 200,
        'headers': {
            'Content-Type': 'application/json'
        },
        'body': json.dumps({
            'blocks': ansblock,
            'response_type': 'in_channel'
        }),
    }



from setuptools import setup, find_packages


def readme(file='', split=False):
    with open(file) as f:
        if split:
            return f.readlines()
        else:
            return f.read()


setup(
    name='Flameboi',
    version='0.0.1dev',
    description='Slack chat bot',
    long_description=readme('README.md'),
    url='https://github.com/ASU-CodeDevils/flameboi-slack-api',
    author='CodeDevil\'s Student Organization',
    author_email='stucampbell.git@gmail.com',
    packages=find_packages(),
    python_requires='>=3.5',
    license='MIT License',
    install_requires=[
        'Flask',
        'python-dotenv',
        'requests',
        'slack',
        'slackclient',
        'slackeventsapi',
    ],
    classifiers=[
        'Topic :: Communications',
        'Topic :: Communications :: Chat',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',

    ],
)



from flameboi.blocks.layout_block.layout_block import LayoutBlock


class DividerBlock(LayoutBlock):
    """
    Defines a divider block. For more information, see: https://api.slack.com/reference/block-kit/blocks#divider
    """

    def __init__(self, block_id: str = None):
        super().__init__(btype='divider', block_id=block_id)



import os
import json
import urllib.request

from base64 import b64decode
from urllib.parse import urlparse

EXPECTED_TOKEN = os.environ['VERIFICATION_TOKEN']

def respond(err, resp_type, block=None):
    return {
        'statusCode': '400' if err else '200',
        'headers': {
            'Content-Type': 'application/json',
        },
        'body': err.message if err else json.dumps({
            'blocks': block,
            'response_type': resp_type 
        })
    }


def lambda_handler(event, context):
    
    params = urllib.parse.parse_qs(b64decode(event['body']))
    
    token = params[b'token'][0].decode("utf-8")
    
    if token != EXPECTED_TOKEN:
        return respond(Exception('Invalid request token'))
        

    block = [
    	{
            "type": "divider"
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": "<https://open.spotify.com/playlist/0lHeOLHPtj3dZnI7hjbEiy?si=vsB1ABcPQZWLDfELbbcPSw|:partywizard:*CodeDevils:partywizard:*>"
            }
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": "<https://open.spotify.com/playlist/1d8nCeom1L86vGT2JgSkzN?si=TjIS1iVcTPqIaGwocuaywA|:partywizard:*ReggaeDevils*:partywizard:>"
            }
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": "<https://open.spotify.com/playlist/37i9dQZF1DWXLeA8Omikj7?si=kdJfP56gS1mAPYEcsOc5fg|*Brain Food*>"
            }
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": "<https://open.spotify.com/playlist/37i9dQZF1DWWQRwui0ExPn?si=SB-cPgIsTL-gvA9xWH3rrg|*LoFi Beats*>"
            }
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": "<https://open.spotify.com/playlist/37i9dQZF1DWZeKCadgRdKQ?si=1VgEAzJmRF-nJ0qNRPd-1A|*Deep Focus*>"
            }
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": "<https://open.spotify.com/playlist/1VWJjbN7LBzM6M4FG76ImT|*Concentrate Your Mind / Asian LoFi Hip Hop*>"
            }
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": "<https://open.spotify.com/playlist/08fTEpRNEn0FcyXOWZ60wO?si=xCjkW151TNuhsyjHswqlfQ|*Acid Jazz/Lounge/Dub/Triphop*>"
            }
        }
    ]


    if b'text' in params:
        com = params[b'text'][0].decode("utf-8")
        stripped = com.split(' ', 1)
        if 'privat' in stripped[0]:
            return respond(None, 'ephemeral', block)
    else:
        return respond(None, 'in_channel', block)



from flameboi.blocks.block import Block
from flameboi.blocks.composition_object.text_object import TextObject
from flameboi.blocks.composition_object.option_object import OptionObject


class OptionGroupObject(Block):
    """
    Provides a way to group options in a select menu or multi-select menu. For more information, see:
    https://api.slack.com/reference/block-kit/composition-objects#option_group
    """

    def __init__(self, label: TextObject, options: list):
        # validate input
        label.validate_text_block(max_length=75, required_type=TextObject.BTYPE_PLAIN_TEXT)
        if not isinstance(options[0], OptionObject):
            raise AttributeError('options list needs to be a list of OptionObject objects')
        if len(options) > 100:
            raise AttributeError('options list cannot exceed 100 options')

        self.label = label
        self.options = options

    @staticmethod
    def expand_options_group(options: [OptionObject]) -> list:
        ret = []
        for option in options:
            ret.append(option.render())
        return ret

    def render(self) -> dict:
        return {
            'label': self.label.render(),
            'options': self.expand_options_group(self.options)
        }


from flameboi.blocks.block import Block


class LayoutBlock(Block):
    """
    Defines a basic layout block. For more information, see: https://api.slack.com/reference/block-kit/blocks
    """

    def __init__(self, btype: str, block_id: str = None):
        if block_id and len(block_id) > 255:
            raise AttributeError(f'block_id cannot be greater than 255 characters, but is {block_id}')

        self.btype = btype
        self.block_id = block_id

    def render(self) -> dict:
        block = {
            'type': self.btype
        }

        if self.block_id:
            block.update({'block_id': self.block_id})

        return block
